/*
 * property list header file.
 *
 * G. Robert Malan (rmalan@eecs.umich.edu)
 * 2/20/97
 *
 * $Id: propertyList.h,v 1.1.1.1 2000/08/14 18:46:16 labovit Exp $
 */

#ifndef _PROPERTY_H_
#define _PROPERTY_H_ 1

/***********************************************************************
 * Some Well-Known Properties:
 ***********************************************************************/

#define NAME_PROPERTY		"NAME"
#define TIMESTAMP_PROPERTY	"TIMESTAMP"
#define COOKIE_PROPERTY		"COOKIE"

/***********************************************************************
 * System properties:
 ***********************************************************************/

#define OBJ_SIZE_PROPERTY	"OBJ_SIZE"
#define	COMMAND_PROPERTY	"COMMAND"

/***********************************************************************
 * Supplier and Client commands (COMMAND_PROPERTY value's):
 ***********************************************************************/

/*
 * Client to server messages:
 */
#define PUBLISH_COMMAND	"PUBLISH"

#define QUERY_COMMAND	"QUERY"
#define REMOVE_QUERY	"REMOVE QUERY"

#define SUPPLY_COMMAND	"SUPPLY"
#define REMOVE_SUPPLY 	"REMOVE SUPPLY"
#define MATCH_SUPPLIES	"MATCH SUPPLIES"

#define SETUP_CONDOLENCE	"SETUP CONDOLENCE"
#define UNICAST_MESSAGE	"UNICAST MESSAGE"
#define SOURCE		"SOURCE"
#define DESTINATION	"DESTINATION"

/*
 * Server to client messages:
 */
#define QUERY_RESPONSE	"QUERY RESPONSE"
#define SUPPLY_RESPONSE	"SUPPLY RESPONSE"

#define QUERY_EXPIRED	"QUERY EXPIRED"
#define SUPPLIERS		"SUPPLIERS"

#define NOTIFICATION	"NOTIFICATION"

/***********************************************************************
 * Query/Supply Properties and values:
 ***********************************************************************/

#define QUERY_ID		"QUERY ID"
#define QUERY_FAILED	"QUERY FAILED"

#define SUPPLY_ID		"SUPPLY ID"
#define SUPPLY_FAILED	"SUPPLY FAILED"

/***********************************************************************
 * Administrative commands (COMMAND_PROPERTY values's):
 ***********************************************************************/

#define ENUM_CONNECTIONS	"ENUMERATE CONNECTIONS"
#define ENUM_CONNECTIONS_RESPONSE "ENUMERATE CONNECTIONS RESPONSE"

#define ENUM_QUEUES	"ENUMERATE QUEUES"
#define ENUM_QUEUES_RESPONSE	 "ENUMERATE QUEUES RESPONSE"

#define DUMP_QUEUE	"DUMP QUEUE"
#define DUMP_QUEUE_RESPONSE	 "DUMP QUEUE RESPONSE"

#define REMOTE_CONNECT	"REMOTE CONNECT"
#define REMOTE_CONNECT_RESPONSE	"REMOTE CONNECT RESPONSE"
#define HOSTNAME_PROPERTY		"HOSTNAME"

#define LOSE_TAIL			"LOSE TAIL"

#define SET_MAX_QUEUE_SIZE		"SET MAX QUEUE SIZE"
#define SET_MAX_QUEUE_SIZE_RESPONSE	"SET MAX QUEUE SIZE RESPONSE"
#define SET_CUR_QUEUE_SIZE		"SET CUR QUEUE SIZE"
#define SET_CUR_QUEUE_SIZE_RESPONSE	"SET CUR QUEUE SIZE RESPONSE"
#define QUEUE_PROPERTY		"QUEUE"

#define REMOVE_CONNECTION		"REMOVE CONNECTION"

/***********************************************************************
 * Data struct for current implementation.
 ***********************************************************************/

typedef struct plist_entry {
	char *key;
	char *value;
	struct plist_entry *previous;
	struct plist_entry *next;
} *plist_entry_t;

typedef struct plist {
	plist_entry_t head;
} *plist_t;

/***********************************************************************
 * Prototypes:
 ***********************************************************************/

/* Interface 1.0+ routines: */
plist_t createPropertyList(void);
void destroyPropertyList(plist_t plist);
int updateProperty(plist_t plist, char *key, char *value);
plist_t copyPropertyList(plist_t from);
void deleteProperty(plist_t list, char *key);
char *getProperty(plist_t plist, char *key);
int iteratePropertyList(plist_t plist, plist_entry_t * cookie,
		char **key, char **value);
int plistEqual(plist_t one, plist_t two);
void dumpPropertyList(plist_t plist);

/* Interface 2.0 routines: */
char *marshallPlist(plist_t plist, unsigned long *length);
plist_t unmarshallPlist(char *mashalledPlist, unsigned long plist_length);

#endif /* _PROPERTY_H_ */
