#include <errno.h>
#include <stdio.h>
#include <string.h>
#define __FAVOR_BSD /*XXX Linux reqs this */
#include "trace.h"

/*
 * Return: -3 on timeout
 *       -2 on ICMP time exceeded in transit (caller keeps going)
 *       -1 on ICMP port unreachable (caller is done)
 *     >= 0 return value is some other ICMP unreachable code
 *
 * I needed to change this function from being signal-based IO,
 * to select or whatnot.
 */

int
recv_v4(Proto *pr, int seq, struct timeval *tv, char *recvbuf,
		int datalen, int verbose)
{
	int hlen1, hlen2, icmplen;
	int len;
	ssize_t n;
	struct ip *ip, *hip;
	struct icmp *icmp;
	struct udphdr *udp;
	fd_set	recvFDSet;
	struct timeval TimeOut;

	FD_ZERO(&recvFDSet);
	TimeOut.tv_sec  = 3;
	TimeOut.tv_usec = 0;

	do {
		len = pr->salen;
		FD_SET(pr->recvfd, &recvFDSet);
		n = select(pr->recvfd + 1, &recvFDSet, NULL, NULL,
			&TimeOut);
//		fprintf(stdout, "recv_v4: select returned %d\n", n);
		if (n == -1) {
			// Need to modify return code or something
			return -2;
		}
		else if (n == 0) {
			//Timed out
			return -3;
		}
		else {
			//Data is ready
			n = recvfrom(pr->recvfd, recvbuf,
				BUFSIZE + datalen, 0, pr->sarecv, &len);
			if (n < 0) {
				if (errno == EINTR) {
					// alarm expired
					return (-3);
				}
				else {
					// error
					return(-4);
				}
			}
		}

		gettimeofday(tv, NULL);	  // get time of packet arrival

		// start of IP header
		ip = (struct ip *) recvbuf;
		// length of IP header
		hlen1 = ip->ip_hl << 2;

		// start of ICMP header
		icmp = (struct icmp *) (recvbuf + hlen1);
		if ((icmplen = n - hlen1) < 8) {
//			err_quit("icmplen (%d) < 8", icmplen);
			return -5;
		}

		if ((icmp->icmp_type == ICMP_TIMXCEED) &&
			(icmp->icmp_code == ICMP_TIMXCEED_INTRANS)) {
			if (icmplen < 8 + 20 + 8) {
//				err_quit("icmplen (%d) < 8 + 20 + 8", icmplen);
				return -5;
			}

			hip = (struct ip *) (recvbuf + hlen1 + 8);
			hlen2 = hip->ip_hl << 2;
			udp = (struct udphdr *) (recvbuf + hlen1 + 8 + hlen2);
			if (hip->ip_p == IPPROTO_UDP &&
				udp->uh_sport == htons(pr->sport) &&
				udp->uh_dport == htons(pr->dport + seq)) {
				/* we hit an intermediate router */
				return (-2);
			}

		}
		else if (icmp->icmp_type == ICMP_UNREACH) {
			if (icmplen < 8 + 20 + 8) {
//				err_quit("icmplen (%d) < 8 + 20 + 8", icmplen);
				return -5;
			}

			hip = (struct ip *) (recvbuf + hlen1 + 8);
			hlen2 = hip->ip_hl << 2;
			udp = (struct udphdr *) (recvbuf + hlen1 + 8 + hlen2);
			if (hip->ip_p == IPPROTO_UDP &&
				udp->uh_sport == htons(pr->sport) &&
				udp->uh_dport == htons(pr->dport + seq)) {
				if (icmp->icmp_code == ICMP_UNREACH_PORT)
					// reached destination
					return (-1);
				else
					// 0, 1, 2, ...
					return (icmp->icmp_code);
			}
		}
		else if (verbose) {
/*
			printf(" (from %s: type = %d, code = %d)\n",
			inet_ntop(AF_INET, NULL, pr->sarecv, pr->salen),
				icmp->icmp_type, icmp->icmp_code);
*/
		}
		/* Some other ICMP error, recvfrom() again */
	} while(1);
}
